#!/bin/bash

set -ex

source charms.reactive.sh

ARCHIVE_DIR=$CHARM_DIR/files/archives
SYMPHONY_INSTALL_PATH=/opt/ibm/spectrumsymphony
CLUSTER_ADMIN=egoadmin
hostname_node=`hostname`
ip_node=`unit-get private-address`
ARCHITECTURE=`uname -m`


@when_not 'ibm-spectrum-symphony-node.nfsclient.installed'
function install_nfsclient(){
	juju-log "IBM Spectrum Symphony: Pre-reqs will be installed."
	# To check for unsupported platforms
	if [ "$ARCHITECTURE" != "x86_64" -a "$ARCHITECTURE" != "ppc64le" ]; then
	juju-log "IBM Spectrum Symphony: Unsupported platform. IBM Spectrum Symphony installed with this Charm supports only the x86_64 platform and POWER LE (ppc64le) platforms."
	return 1
	fi
	# Install nfs-common using apt
	apt-get install nfs-common -y
	set_state 'ibm-spectrum-symphony-node.nfsclient.installed'
	remove_state 'ibm-spectrum-symphony-node.nfsclient.ready'
	remove_state 'ibm-spectrum-symphony-node.conf.ready'
	juju-log "IBM Spectrum Symphony : Pre-reqs installed successfully"
}

function start_symphony(){
	. $SYMPHONY_INSTALL_PATH/profile.platform
	egosh ego start
	if [ "$?" != "0" ]; then
		juju-log "IBM Spectrum Symphony : Error while starting  Symphony Cluster"
	else
		juju-log "IBM Spectrum Symphony : Symphony Cluster started successfully"
	fi
}


function add_hostname(){
	juju-log "IBM Spectrum Symphony : Symphony node Hostname is : $hostname_node"
	juju-log "IBM Spectrum Symphony : Master host IP is : $ip_node"
	# Adding hostname entry in hosts file
	cd /etc
	if grep -w "$ip_node $hostname_node" hosts
	then
		juju-log "IBM Spectrum Symphony : Host file already updated"
	else
		juju-log "IBM Spectrum Symphony : Updating Host file"
		echo "$ip_node $hostname_node" >> /etc/hosts
	fi
}


# Function to find symphony version and distribution type

function find_version_distr()
{
	# To find the symphony version
	ps_version=`grep "EGO_VERSION" $SYMPHONY_INSTALL_PATH/kernel/conf/ego.conf | awk -F'=' '{print $2}'`

	ARCHITECTURE=`uname -m`
	cd $SYMPHONY_INSTALL_PATH/$ps_version
	if [ "$ARCHITECTURE" = "x86_64" ]; then
		distribution_OS_type=`ls -lrt | grep x86_64 | awk -F' ' '{print $9}'`
	elif [ "$ARCHITECTURE" = "ppc64le" ]; then
		distribution_OS_type=`ls -lrt | grep ppc64le | awk -F' ' '{print $9}'`
	fi
}


# Block of code for sharing the Symphony installation path with other hosts which are part of Symphony cluster using NFS.
function nfs_sharesymphony(){
	# Using NFS , share the Symphony file location installed on storage server and mount it on master host
	ip_storage=$1
	juju-log "IBM Spectrum Symphony : IP storage is : $ip_storage"
	if  [ -d $SYMPHONY_INSTALL_PATH ] ; then
		juju-log "Symphony Directory already exists"
	else
		mkdir -p $SYMPHONY_INSTALL_PATH
	fi

	if grep -q "$ip_storage:$SYMPHONY_INSTALL_PATH" /etc/fstab
	then
		juju-log "IBM Spectrum Symphony: Entries already exist in /etc/fstab for mounting"
	else
		echo "$ip_storage:$SYMPHONY_INSTALL_PATH $SYMPHONY_INSTALL_PATH nfs rw 0 0" >> /etc/fstab
	fi

	mount -a || true
	if [  -d $SYMPHONY_INSTALL_PATH/kernel/conf ]; then
		juju-log "IBM Spectrum Symphony : Successfully mounted the Symphony Directory"
	else
		juju-log "IBM Spectrum Symphony : Error while mounting the Symphony Directory, check the /etc/fstab or check whether the Storage Server is ready."
		return 0
	fi
}


function createuser_conf(){
	# checking whether egoadmin already exists or not
	cd /etc
	if grep -q "egoadmin" passwd
	then
		juju-log "IBM Spectrum Symphony : User egoadmin already exists"
	else
		useradd -m -d /home/$CLUSTER_ADMIN $CLUSTER_ADMIN
		if [ $? == 0 ]; then
			juju-log "IBM Spectrum Symphony : Symphony administrator egoadmin user created successfully"
		else
			juju-log "IBM Spectrum Symphony: Error while creating the symphony administrator user "
			return 0
		fi
	fi

	find_version_distr
	cd /etc
	if [[ "`grep "EGO_STARTUP_USERS" ego.sudoers`" ]] && [[ "`grep "EGO_STARTUP_PATH" ego.sudoers`" ]]; then
		juju-log "IBM Spectrum Symphony : ego.sudoers file already updated with values"
	else
		touch ego.sudoers
		echo "EGO_STARTUP_PATH=$SYMPHONY_INSTALL_PATH/$ps_version/$distribution_OS_type/etc" >> ego.sudoers
		sed -i '1i EGO_STARTUP_USERS="egoadmin"' ego.sudoers
		chmod 600 ego.sudoers
		juju-log "IBM Spectrum Symphony: ego.sudoers file updated."
	fi
}


@when_not 'storage.joined' 'ibm-spectrum-symphony-node.nfsclient.ready'
function notify_user_storage(){
	juju-log "IBM Spectrum Symphony : Please add a relation between Symphony Storage and Symphony Master charm to have a working Symphony Master"
	status-set waiting "Waiting for relation with Symphony Storage/Details not sent from Storage Server"
}


# When relation is established between Symphony Storage and Symphony node
@when 'storage.available'
@when_not 'ibm-spectrum-symphony-node.nfsclient.ready'
function configure_symphonyserver(){
	juju-log "IBM Spectrum Symphony : Getting connected to Storage Server and mounting of shared Symphony installation files"
	ip=$(relation_call --state=storage.available private_address) || true
	juju-log "IBM Spectrum Symphony : IP of Symphony Storage Server as $ip"
	hostname_storage=$(relation_call --state=storage.available get_hostname) || true
	juju-log "IBM Spectrum Symphony : Hostname of Symphony Storage Server as $hostname_storage"
	add_hostname
	nfs_sharesymphony $ip
	createuser_conf
	# Set state that now NFS client is ready and files are mounted.
	juju-log "IBM Spectrum Symphony : NFS Client is ready now and Symphony Files have been shared."
	status-set maintenance "NFS client is ready and Symphony Files shared"
	set_state 'ibm-spectrum-symphony-node.nfsclient.ready'
}

@when_none 'master.joined' 'master.MasterReConfigReady'
function notify_user(){
	juju-log "IBM Spectrum Symphony : Please add a relation between Symphony node and Symphony Master charm to add the Symphony node to the Symphony Cluster"
	status-set waiting "Waiting for relation with Symphony Master/Details not sent from Symphony Master"
}


# When relation is added between Symphony node and Master, share the hostname of symphony node.
@when 'master.joined'
function send_hostname_master(){
	juju-log "IBM Spectrum Symphony : Sending Symphony node Hostname to Symphony Master"
	relation_call --state=master.joined set_hostname $hostname_node || true
}

# When Symphony Master is ready and reconfigured Symph'master.MasterReConfigReady'ony Daemons and NFS client is also ready, then start the Symphony node Daemons to make the Symphony node up and running.
@when 'master.MasterReConfigReady' 'ibm-spectrum-symphony-node.nfsclient.ready'
function configure_symphonynode_master(){
	count=0
	count1=0
	juju-log "IBM Spectrum Symphony : Configuring Symphony node when Symphony Master is ready"
	ip_master=$(relation_call --state=master.MasterReConfigReady private_address) || true
	juju-log "IBM Spectrum Symphony : IP of Symphony master as $ip_master"
	hostname_master=$(relation_call --state=master.MasterReConfigReady get_hostname) || true
	juju-log "IBM Spectrum Symphony : Hostname of Symphony Master as $hostname_master"

	# First add the host details of Symphony Master in /etc/hosts file before starting the Symphony Deamons.
	for master_ip in $ip_master
	do
		for master_hostname in $hostname_master
		do
			if [ $count == $count1 ]; then
				cd /etc
				if grep -w "$master_ip $master_hostname" hosts
				then
					juju-log "IBM Spectrum Symphony : Host file already updated with host information of the Symphony node"
				else
					juju-log "IBM Spectrum Symphony : Updating Host file"
					echo "$master_ip $master_hostname" >> /etc/hosts
				fi
			fi
		count1=$((count1+1))
		done
		count=$((count+1))
		count1=0
	done
	# Getting the flag value to indicate Symphony Master has reconfigured itself or not.
	reconfigFlag_master=$(relation_call --state=master.MasterReConfigReady notify_server) || true
	for master_flag in $reconfigFlag_master
	do
		if [ "$master_flag" == "MasterReConfigReady" ]; then
			# start the symphony services
			start_symphony
			set_state 'ibm-spectrum-symphony-node.conf.ready'
			status-set active "Symphony node is ready"
		elif [ "$master_flag" == "NotMaster" ]; then
			start_symphony || true
			set_state 'ibm-spectrum-symphony-node.conf.ready'
			status-set active "Symphony node is ready"
		fi
	done
}


# Stop the Symphony services once service is removed or relation is broken
function stop_symphonynode(){
	set +e
	. $SYMPHONY_INSTALL_PATH/profile.platform
	egosh ego shutdown -f
	set -e
}



# When relation between Symphony Master and Symphony node is departed, stop the Symphony Deamons and remove entry from ego.cluster.cluster1 file.
@when 'master.departed' 'ibm-spectrum-symphony-node.conf.ready'
function remove_serverhost(){
	# Stop the Symphony Daemons first
	stop_symphonynode
	# Remove entry from ego.cluster file
	sed -i  '/'$hostname_node'/d' $SYMPHONY_INSTALL_PATH/kernel/conf/ego.cluster.cluster1
	remove_state 'ibm-spectrum-symphony-node.conf.ready'
	relation_call --state=master.departed dismiss_departed || true
}

# When relation between Symphony Storage Server and Symphony node is departed. If relation with Symphony Master exists, first remove that then unmount the Symphony Files.
@when_none 'ibm-spectrum-symphony-node.conf.ready'
@when 'storage.departing' 'ibm-spectrum-symphony-node.nfsclient.ready'
function nfsstorage_depart(){
	remove_serverhost
	mount_flag=`stat -f -L -c %T $SYMPHONY_INSTALL_PATH` || true
	if  [ "$mount_flag" == "ext2/ext3" ]; then
		juju-log "IBM Spectrum Symphony : /opt/ibm/spectrumsymphony directory is not mounted yet, cannot proceed with unmounting"
		return 0
	fi
	if ! [ -d $SYMPHONY_INSTALL_PATH/conf ]; then
		juju-log "IBM Spectrum Symphony : No Symphony folders , nothing to unmount"
	else
		cd /home
		set +e
		umount -f -l $SYMPHONY_INSTALL_PATH 
		set -e
		juju-log " IBM Spectrum Symphony : Unmounted the Symphony installation folders, now Symphony folders will not be accessible"
		sed -i  "\|$SYMPHONY_INSTALL_PATH|d" /etc/fstab     
		remove_state 'ibm-spectrum-symphony-node.nfsclient.ready'
	fi
}

reactive_handler_main
